From a8dbb921e45011e21396fdb7ed1c70e09d446c5b Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?=E4=BD=99=E6=98=8C=E5=B9=B3?= <dead.ash@hotmail.com>
Date: Wed, 17 Jun 2020 13:59:42 +0800
Subject: [PATCH 2/2] kernel32/test: Add tests for getting main thread id 
by toolhelp.
MIME-Version: 1.0
Content-Type: text/plain; charset=UTF-8
Content-Transfer-Encoding: 8bit

Some applications mistake the first thread id as the main thread id.

Signed-off-by: Changping Yu <dead.ash@hotmail.com>
---
 dlls/kernel32/tests/toolhelp.c | 38 ++++++++++++++++++++++++++++++++++
 1 file changed, 38 insertions(+)

diff --git a/dlls/kernel32/tests/toolhelp.c b/dlls/kernel32/tests/toolhelp.c
index 9250f8dc54..08f1dbd9fa 100644
--- a/dlls/kernel32/tests/toolhelp.c
+++ b/dlls/kernel32/tests/toolhelp.c
@@ -150,6 +150,43 @@ static void test_process(DWORD curr_pid, DWORD sub_pcs_pid)
     ok(!pProcess32First( hSnapshot, &pe ), "shouldn't return a process\n");
 }
 
+static void test_main_thread(DWORD curr_pid, DWORD curr_tid)
+{
+    HANDLE              hSnapshot;
+    THREADENTRY32       te;
+    HANDLE              ev, hThread;
+    DWORD               tid, main_tid = 0;
+
+    ev = CreateEventW(NULL, FALSE, FALSE, NULL);
+    ok(ev != NULL, "Cannot create event\n");
+
+    hThread = CreateThread(NULL, 0, sub_thread, ev, 0, &tid);
+    ok(hThread != NULL, "Cannot create thread\n");
+
+    hSnapshot = pCreateToolhelp32Snapshot( TH32CS_SNAPTHREAD, 0 );
+    ok(hSnapshot != NULL, "Cannot create snapshot\n");
+
+    /* check that this current process is enumerated */
+    te.dwSize = sizeof(te);
+    if (pThread32First( hSnapshot, &te ))
+    {
+        do
+        {
+            if (te.th32OwnerProcessID == curr_pid){
+                main_tid = te.th32ThreadID;
+                break;
+            }
+        } while (pThread32Next( hSnapshot, &te ));
+    }
+
+    ok(curr_tid == main_tid, "check main thread is error\n");
+
+    SetEvent(ev);
+    CloseHandle(hThread);
+    CloseHandle(ev);
+    CloseHandle(hSnapshot);
+}
+
 static void test_thread(DWORD curr_pid, DWORD sub_pcs_pid)
 {
     HANDLE              hSnapshot;
@@ -339,6 +376,7 @@ START_TEST(toolhelp)
 
     test_process(pid, info.dwProcessId);
     test_thread(pid, info.dwProcessId);
+    test_main_thread(pid, GetCurrentThreadId());
     test_module(pid, curr_expected_modules, ARRAY_SIZE(curr_expected_modules));
     test_module(info.dwProcessId, sub_expected_modules, ARRAY_SIZE(sub_expected_modules));
 
-- 
2.26.1.windows.1

